#!/usr/bin/env python

"""
Combine a set of GVCFs into a MatrixTable
"""

import os
from typing import List, Optional
import logging
import shutil

import click
import pandas as pd

import hail as hl

from cpg_utils import to_path

from hail.experimental.vcf_combiner import vcf_combiner
from hail.experimental.vcf_combiner.vcf_combiner import CombinerConfig

from larcoh.utils import exists, check_duplicates


logger = logging.getLogger('combine_gvcfs')
logger.setLevel('INFO')


# The target number of rows per partition during each round of merging
TARGET_RECORDS = 25_000

# Default number of partitions in a matrix table generated by combiner
MAX_PARTITIONS = 2586


@click.command()
@click.option(
    '--sample-gvcf-tsv',
    'sample_gvcf_tsv_path',
    required=True,
    help='Sample data TSV path, to get samples names from the "s" column',
)
@click.option(
    '--out-mt',
    'out_mt_path',
    required=True,
    help='path to write the combined MatrixTable',
)
@click.option(
    '--branch-factor',
    'branch_factor',
    required=False,
    type=click.INT,
    default=CombinerConfig.default_branch_factor,
    help='Combiner branch factor (gvcfs will be split into ~branch_factor batches)',
)
@click.option(
    '--batch-size',
    'batch_size',
    required=False,
    type=click.INT,
    default=CombinerConfig.default_phase1_batch_size,
    help='Combiner batch size for phase1 (size of each batch of gvcfs to process '
         'in 1 job)',
)
@click.option(
    '--existing-mt',
    'existing_mt_path',
    help='optional path to an existing MatrixTable. '
    'If provided, will be read and used as a base to get extended with the '
    'samples in the input sample map. Can be read-only, as it will not '
    'be overwritten, instead the result will be written to the new location '
    'provided with --out-mt',
)
@click.option(
    '--tmp-bucket',
    'tmp_bucket',
    required=True,
    help='path to folder for intermediate output. '
    'Can be a Google Storage URL (i.e. start with `gs://`).',
)
@click.option(
    '--local-tmp-dir',
    'local_tmp_dir',
    help='local directory for temporary files and Hail logs (must be local).',
)
@click.option(
    '--n-partitions',
    'n_partitions',
    type=click.INT,
    help='Number of partitions for the output matrix table',
)
@click.option(
    '--is-exome',
    'is_exome',
    is_flag=True,
    default=False,
)
def main(
    sample_gvcf_tsv_path: str,
    out_mt_path: str,
    branch_factor: int,
    batch_size: int,
    existing_mt_path: Optional[str],
    tmp_bucket: str,
    local_tmp_dir: str,
    n_partitions: Optional[int],
    is_exome: bool,
):  # pylint: disable=missing-function-docstring
    if n_partitions:
        if n_partitions >= 2586:
            raise click.BadParameter(
                f'n_partitions must be less than {MAX_PARTITIONS} (combiner\'s default '
                f'number of partitions), got {n_partitions}'
            )

    hl.init(default_reference='GRCh38')
    logger.info(f'Combining GVCFs. Reading input paths from {sample_gvcf_tsv_path}')
    with to_path(sample_gvcf_tsv_path).open() as f:
        df = pd.read_table(f)
    sample_names = df.s
    gvcf_paths = df.gvcf
    check_duplicates(sample_names)
    check_duplicates(gvcf_paths)
    print(f'Combining {len(sample_names)} samples: {", ".join(sample_names)}')

    if n_partitions is not None or existing_mt_path is not None:
        # We are going to re-partition later, so writing the
        # matrix table into tmp:
        new_mt_path = os.path.join(tmp_bucket, 'new.mt')
    else:
        new_mt_path = out_mt_path

    if not exists(new_mt_path):
        combine_gvcfs(
            gvcf_paths=gvcf_paths,
            sample_names=sample_names,
            out_mt_path=new_mt_path,
            tmp_bucket=tmp_bucket,
            branch_factor=branch_factor,
            batch_size=batch_size,
            is_exome=is_exome,
        )
    new_mt = hl.read_matrix_table(new_mt_path)
    _log_mt_write(new_mt, new_mt_path)

    new_plus_existing_mt = None
    if existing_mt_path:
        logger.info(f'Combining with the existing matrix table {existing_mt_path}')
        new_plus_existing_mt = _combine_with_the_existing_mt(
            existing_mt_path=existing_mt_path,
            new_mt_path=new_mt_path,
        )
        
    mt = new_plus_existing_mt or new_mt
    if n_partitions is not None:
        mt = mt.repartition(n_partitions)

    # We need to write only if we repartitioned or combined with existing.
    # Otherwise, we are happy with the initial matrix table.
    if existing_mt_path or n_partitions is not None:
        mt.write(out_mt_path, overwrite=True)
        _log_mt_write(mt, out_mt_path)

    shutil.rmtree(local_tmp_dir)
    
    
def _log_mt_write(mt, path):
    logger.info(
        f'Written {mt.cols().count()} new samples to {path}, '
        f'n_partitions={mt.n_partitions()}'
    )


def _combine_with_the_existing_mt(
    existing_mt_path: str,
    # passing as a path because we are going
    # to re-read it with different intervals
    new_mt_path: str,
) -> hl.MatrixTable:
    # Making sure the tables are keyed by locus only, to make the combiner work.
    existing_mt = hl.read_matrix_table(existing_mt_path).key_rows_by('locus')
    intervals = vcf_combiner.calculate_new_intervals(
        existing_mt.rows(),
        n=TARGET_RECORDS,
        reference_genome='GRCh38',
    )
    new_mt = hl.read_matrix_table(new_mt_path, _intervals=intervals).key_rows_by(
        'locus'
    )
    logger.info(
        f'Combining {new_mt_path} ({new_mt.count_cols()} samples) '
        f'with an existing MatrixTable {existing_mt_path} '
        f'({existing_mt.count_cols()} samples), '
        f'split into {existing_mt.n_partitions()} partitions)'
    )
    out_mt = vcf_combiner.combine_gvcfs([existing_mt, new_mt])
    return out_mt


def combine_gvcfs(
    gvcf_paths: List[str],
    sample_names: List[str],
    out_mt_path: str,
    tmp_bucket: str,
    branch_factor: int,
    batch_size: int,
    is_exome: bool = False,
):
    """
    Combine a set of GVCFs in one go
    """
    hl.experimental.run_combiner(
        gvcf_paths,
        sample_names=sample_names,
        # header must be used with sample_names, otherwise sample_names will be ignored
        # first gvcf path is fine as a header because it will be read until 
        # the last line starting with # 
        header=gvcf_paths[0],
        out_file=out_mt_path,
        reference_genome='GRCh38',
        use_genome_default_intervals=not is_exome,
        use_exome_default_intervals=is_exome,
        tmp_path=tmp_bucket,
        overwrite=True,
        key_by_locus_and_alleles=True,
        branch_factor=branch_factor,
        batch_size=batch_size,
    )


if __name__ == '__main__':
    main()  # pylint: disable=E1120



#!/usr/bin/env python

"""
Convert annotated site-only table to a sites-only VCF.
Essentially a verbatim copy of: hail-ukbb-200k-callset:mt_to_vcf.py
"""

import logging
import click
import hail as hl
from gnomad.utils.vcf import adjust_vcf_incompatible_types
from gnomad.utils.sparse_mt import default_compute_info
from joint_calling import _version
from joint_calling.utils import get_validation_callback, init_hail, file_exists
from joint_calling import utils

logger = logging.getLogger('vqsr_qc')
logging.basicConfig(
    format='%(asctime)s (%(name)s %(lineno)s): %(message)s',
    datefmt='%m/%d/%Y %I:%M:%S %p',
)
logger.setLevel(logging.INFO)


@click.command()
@click.version_option(_version.__version__)
@click.option(
    '--ht',
    'ht_path',
    required=True,
    callback=get_validation_callback(ext='ht', must_exist=True),
    help='path to annotated Hail table',
)
@click.option(
    '-o',
    'output_path',
    required=True,
)
@click.option(
    '--local-tmp-dir',
    'local_tmp_dir',
    help='local directory for temporary files and Hail logs (must be local).',
)
@click.option(
    '--overwrite/--reuse',
    'overwrite',
    is_flag=True,
    help='if an intermediate or a final file exists, skip running the code '
    'that generates it.',
)
@click.option(
    '--hail-billing',
    'hail_billing',
    help='Hail billing account ID.',
)
def main(
    ht_path: str,
    output_path: str,
    local_tmp_dir: str,
    overwrite: bool,
    hail_billing: str,  # pylint: disable=unused-argument
):  # pylint: disable=missing-function-docstring
    init_hail(__file__, local_tmp_dir)
    
    if utils.can_reuse(output_path, overwrite):
        return
    
    logger.info(
        f'Exporting sites-only VCF to {output_path} to run in the VQSR pipeline'
    )
    hl.export_vcf(hl.read_table(ht_path), output_path)
    logger.info('Successfully exported sites-only VCF')


if __name__ == '__main__':
    main()  # pylint: disable=E1120



#!/usr/bin/env python

"""
Generate a subset Matrix Table given a list of projects.

Will also remove reference blocks.
"""

import logging
from os.path import join
from typing import Optional, List

import click
import hail as hl

from joint_calling.utils import get_validation_callback
from joint_calling import utils, _version

logger = logging.getLogger('joint-calling')
logger.setLevel('INFO')


@click.command()
@click.version_option(_version.__version__)
@click.option(
    '--mt',
    'mt_path',
    required=True,
    callback=get_validation_callback(ext='mt', must_exist=True),
    help='path to final filtered split matrix table',
)
@click.option(
    '--subset-project',
    'subset_projects',
    multiple=True,
    required=True,
    help='Create a subset matrix table including only these projects'
)
@click.option(
    '--out-mt',
    'out_mt_path',
    required=True,
    callback=get_validation_callback(ext='mt', must_exist=False),
    help='path to write the subset matrix table',
)
@click.option(
    '--local-tmp-dir',
    'local_tmp_dir',
    help='local directory for temporary files and Hail logs (must be local).',
)
@click.option(
    '--hail-billing',
    'hail_billing',
    help='Hail billing account ID.',
)
def main(
    mt_path: str,
    subset_projects: List[str],
    out_mt_path: str,
    local_tmp_dir: str,
    hail_billing: str,  # pylint: disable=unused-argument
):  # pylint: disable=missing-function-docstring

    utils.init_hail(__file__, local_tmp_dir)

    # This will return a table with split multiallelics, and thus GT as a field
    mt = hl.read_matrix_table(mt_path)
    all_projects = list(set(mt.meta.project.collect()))
    logger.info(
        f'Full matrix table: {mt.count_cols()} samples, '
        f'{mt.count_rows() / 1_000_000}M rows, '
        f'{len(all_projects)} projects: {", ".join(all_projects)}'
    )

    subset_projects = list(set(subset_projects))
    diff_projects = set(subset_projects) - set(all_projects)
    if diff_projects:
        raise click.BadParameter(
            f'--subset-project values should be a subset of the existing matrix '
            f'table projects: mt.meta.project ({all_projects}). '
            f'The following projects are not in existing projects: {diff_projects} '
        )
    logger.info(f'Subsetting to {", ".join(subset_projects)}')

    mt = mt.filter_cols(hl.literal(subset_projects).contains(mt.meta.project))
    mt = mt.filter_rows(hl.agg.any(mt.GT.is_non_ref()))
    
    mt.write(out_mt_path, overwrite=True)
    mt = hl.read_matrix_table(out_mt_path)
    new_projects = list(set(mt.meta.project.collect()))
    logger.info(
        f'Subset matrix table: {mt.count_cols()} samples, {mt.count_rows()} rows, '
        f'{len(all_projects)} projects: {", ".join(new_projects)}'
    )


if __name__ == '__main__':
    main()  # pylint: disable=E1120



import collections
from typing import Callable
import logging

from cpg_utils import Path, to_path
from cpg_utils.config import get_config
from cpg_utils.hail_batch import dataset_path
from hailtop.batch.job import Job

from larcoh.hb.batch import setup_batch
from larcoh.providers.cpg.inputs import CpgInputProvider
from larcoh.providers.cpg.metamist import Metamist
from larcoh.targets import Cohort
from larcoh.filetypes import SequencingType
from larcoh.utils import can_reuse, timestamp, slugify

logger = logging.getLogger(__file__)


class Pipeline:
    def __init__(self):
        """
        Create pipeline.
        """
        self.output_version = get_config()['workflow']['output_version']
        self.analysis_prefix = to_path(
            dataset_path(f'joint-calling/{self.output_version}', category='analysis')
        )

        run_id = get_config()['workflow'].get('run_id', timestamp())
        analysis_dataset = get_config()['workflow']['dataset']
        name = get_config()['workflow'].get('name')
        description = get_config()['workflow'].get('description')
        name = name or description or analysis_dataset
        name = slugify(name)
        description = description or name
        description += f': run_id={run_id}'

        self.cohort = Cohort(
            analysis_dataset_name=analysis_dataset,
            name=name,
            sequencing_type=SequencingType.parse(
                get_config()['workflow']['sequencing_type']
            ),
        )

        CpgInputProvider(
            db=Metamist(self.cohort.analysis_dataset.name)
        ).populate_cohort(
            cohort=self.cohort,
            dataset_names=get_config()['workflow'].get('datasets'),
            skip_samples=get_config()['workflow'].get('skip_samples'),
            only_samples=get_config()['workflow'].get('only_samples'),
            skip_datasets=get_config()['workflow'].get('skip_datasets'),
        )
        if get_config()['workflow'].get('access_level') == 'test':
            subset_to_test(self.cohort)

        logger.info(
            f'Using {len(self.cohort.get_samples())} samples '
            f'from {len(self.cohort.get_datasets())} datasets'
        )

        self.tmp_prefix = self.cohort.analysis_dataset.tmp_prefix() / run_id

        description += f' [{self.cohort.sequencing_type.value}]'
        if ds_set := set(d.name for d in self.cohort.get_datasets()):
            description += ' ' + ', '.join(sorted(ds_set))
        self.b = setup_batch(description=description)
        self.tasks = []

    def add_task(self, task: 'Task'):
        task.work_fn(task)

    def create_task(
        self,
        name: str,
        work_fn: Callable[['Task'], list[Job]],
        inputs: list[Path] | None = None,
        outputs: list[Path] | None = None,
    ):
        self.add_task(
            Task(
                name=name,
                pipeline=self,
                work_fn=work_fn,
                inputs=inputs,
                outputs=outputs,
            )
        )

    def run(self):
        self.b.run()


class Task:
    def __init__(
        self,
        name: str,
        work_fn: Callable[['Task'], list[Job]],
        pipeline: 'Pipeline',
        inputs: list[Path] | None = None,
        outputs: list[Path] | None = None,
    ):
        self.name = name
        self.work_fn = work_fn
        self.inputs = inputs or []
        self.outputs = outputs or []
        self.pipeline = pipeline

    def run(self) -> list[Job]:
        overwrite = get_config()['workflow'].get('overwrite', False)
        if self.outputs and can_reuse(self.outputs, overwrite):
            return [self.pipeline.b.new_job(f'{self.name} [reuse]')]
        return self.work_fn(self)


def subset_to_test(cohort: Cohort):
    """
    Function that subsets the cohort for test runs.
    """
    original_sample_cnt = len(cohort.get_samples())
    original_dataset_cnt = len(cohort.get_datasets())
    # Test dataset: collecting all batch1 samples, plus 4 samples from each of other batches
    samples_by_batch: dict[str, list] = collections.defaultdict(list)
    for sample in cohort.get_samples():
        batch = sample.sequencing_meta_by_type[SequencingType.GENOME]['batch']
        if batch != '1' and len(samples_by_batch[batch]) > 4:
            sample.active = False
            continue
        samples_by_batch[batch].append(sample)
    logger.info(
        f'After subset: {len(cohort.get_samples())}/{original_sample_cnt} samples, '
        f'{len(samples_by_batch)} batches, '
        f'{len(cohort.get_datasets())}/{original_dataset_cnt} datasets'
    )



"""
Utility functions and constants.
"""
import collections
import re
import unicodedata
import logging
import string
import sys
import time
import traceback
from functools import lru_cache
from random import choices
from typing import cast

from cpg_utils import Path, to_path

logger = logging.getLogger(__file__)


# Packages to install on a dataproc cluster, to use with the dataproc wrapper.
DATAPROC_PACKAGES = [
    'cpg_utils',
    'larcoh',
    'cpg_gnomad',  # github.com/populationgenomics/gnomad_methods
    'seqr_loader==1.2.5',  # hail-elasticsearch-pipelines
    'elasticsearch==8.1.1',
    'cpg_utils',
    'click',
    'google',
    'fsspec',
    'sklearn',
    'gcloud',
    'selenium',
]


@lru_cache
def exists(
    path: Path | str, verbose: bool = True, description: str | None = None
) -> bool:
    """
    Caching version of the existence check.
    The python code runtime happens entirely during the pipeline submittion,
    without waiting for it to finish, so there is no expectation that object
    existence status would change during the runtime. This, this function uses
    `@lru_cache` to make sure that object existence is checked only once.
    """
    return exists_not_cached(path, verbose)


def exists_not_cached(
    path: Path | str, verbose: bool = True, description: str | None = None
) -> bool:
    """
    Check if the object exists, where the object can be:
        * local file
        * local directory
        * cloud object
        * cloud URL representing a *.mt or *.ht Hail data,
          in which case it will check for the existence of a
          *.mt/_SUCCESS or *.ht/_SUCCESS file.
    @param path: path to the file/directory/object/mt/ht
    @param verbose: print on each check
    @param description: optional string to print in verbose mode
    @return: True if the object exists
    """
    extra_msg = '' if not description else f' ({description})'

    path = cast(Path, to_path(path))

    # rstrip to ".mt/" -> ".mt"
    if any(str(path).rstrip('/').endswith(f'.{suf}') for suf in ['mt', 'ht']):
        path = path / '_SUCCESS'

    if verbose:
        # noinspection PyBroadException
        try:
            res = path.exists()
        except BaseException:
            traceback.print_exc()
            logger.error(f'Failed checking {path}' + extra_msg)
            sys.exit(1)
        logger.debug(
            f'Checked {path} [' + ('exists' if res else 'missing') + ']' + extra_msg
        )
        return res
    return path.exists()


def can_reuse(
    path: list[Path] | Path | str | None,
    overwrite: bool,
) -> bool:
    """
    Checks if `fpath` is good to reuse in the analysis: it exists
    and `overwrite` is False.

    If `fpath` is a collection, it requires all files in it to exist.
    """
    if overwrite:
        return False

    if not path:
        return False

    if isinstance(path, list):
        return all(can_reuse(fp, overwrite) for fp in path)

    if not exists(path):
        return False

    logger.debug(f'Reusing existing {path}. Use --overwrite to overwrite')
    return True


def timestamp(rand_suffix_len: int = 5) -> str:
    """
    Generate a timestamp string. If `rand_suffix_len` is set, adds a short random
    string of this length for uniqueness.
    """
    result = time.strftime('%Y_%m%d_%H%M')
    if rand_suffix_len:
        rand_bit = ''.join(
            choices(string.ascii_uppercase + string.digits, k=rand_suffix_len)
        )
        result += f'_{rand_bit}'
    return result


def slugify(line: str):
    """
    Slugify a string.

    Example:
    >>> slugify(u"Héllø Wörld")
    u"hello-world"
    """

    return re.sub(
        r'[-\s]+',
        '-',
        re.sub(
            r'[^\w\s-]',
            '',
            unicodedata.normalize('NFKD', line).encode('ascii', 'ignore').decode(),
        )
        .strip()
        .lower(),
    )


def check_duplicates(iterable):
    """
    Throws error if input list contains repeated items.
    """
    duplicates = [
        item for item, count in collections.Counter(iterable).items() if count > 1
    ]
    if duplicates:
        raise ValueError(f'Found {len(duplicates)} duplicates: {duplicates}')
    return duplicates



#!/usr/bin/env python

"""
Combine with HGDP-1KG subset of gnomAD v3, and select high-quality QC sites for PCA.

This script will produce two matrix tables:
1. `--out-hgdp-union-mt` - a union of all input dataset and HGDP-1KG samples,
   with rows corresponding to the rows shared with datasets, and fitered to
   high-quality ld-pruned sites best suited for PCA. The table is stipped off of
   all sample and entry annotations but GT, and with the HGDP-1KG sample annotations
   re-added as a `hgdp_1kg_metadata` column.
2. `--out-mt` - the input dataset with rows filtered down to the rows in 
   `--out-hgdp-union-mt` (so only high-quality sites shared with HGDP-1KG).
"""

import logging
from typing import Optional

import click
import hail as hl
from joint_calling import utils
from joint_calling import _version
from joint_calling.resources import ANCESTRY_SITES


logger = logging.getLogger(__file__)
logging.basicConfig(format='%(levelname)s (%(name)s %(lineno)s): %(message)s')
logger.setLevel(logging.INFO)


@click.command()
@click.version_option(_version.__version__)
@click.option(
    '--mt',
    'mt_path',
    required=True,
    callback=utils.get_validation_callback(ext='mt', must_exist=True),
    help='path to the Matrix Table',
)
@click.option(
    '--meta-tsv',
    'meta_tsv_path',
    required=True,
    help='path to a CSV with QC and population metadata for the samples',
)
@click.option(
    '--out-mt',
    'out_mt_path',
    callback=utils.get_validation_callback(ext='mt'),
    help='path to write the Matrix Table after subsetting to selected rows. '
    'The difference with --out-hgdp-union-mt is that it contains only the dataset '
    'samples',
)
@click.option(
    '--is-test',
    'is_test',
    is_flag=True,
    help='subset the gnomAD table to 20 samples',
)
@click.option(
    '--hail-billing',
    'hail_billing',
    required=True,
    help='Hail billing account ID.',
)
def main(  # pylint: disable=too-many-arguments,too-many-locals,missing-function-docstring
    mt_path: str,
    meta_tsv_path: str,
    out_mt_path: Optional[str],
    is_test: bool,  # pylint: disable=unused-argument
    hail_billing: str,  # pylint: disable=unused-argument
):
    local_tmp_dir = utils.init_hail(__file__)

    utils.parse_input_metadata(meta_tsv_path, local_tmp_dir)

    sites_ht = hl.read_table(ANCESTRY_SITES).key_by('locus')

    mt = utils.get_mt(mt_path, passing_sites_only=True)
    mt = hl.experimental.densify(mt)
    mt = mt.select_entries(GT=mt.LGT).select_cols()
    mt = mt.filter_rows(
        (hl.len(mt.alleles) == 2)
        & hl.is_snp(mt.alleles[0], mt.alleles[1])
        & mt.locus.in_autosome()
        & hl.is_defined(sites_ht[mt.locus])
    )
    mt = mt.naive_coalesce(5000)
    mt.checkpoint(out_mt_path, overwrite=True)


if __name__ == '__main__':
    main()  # pylint: disable=E1120



import collections
import logging

from larcoh.pipeline import Pipeline
from larcoh.targets import Cohort
from larcoh.tasks.combiner import Combiner
from larcoh.filetypes import SequencingType

logger = logging.getLogger(__file__)


def main():
    pipeline = Pipeline()

    pipeline.add_task(Combiner(pipeline))

    # relatedness_prefix = pipeline.analysis_prefix / 'relatedness'
    #
    # sample_qc_job, hard_filter_ht_path, meta_ht_path = add_sample_qc_jobs(
    #     b=b,
    #     mt_path=raw_combined_mt_path,
    #     samples_tsv_path=samples_tsv_path,
    #     sample_qc_bucket=join(analysis_bucket, 'sample_qc'),
    #     ancestry_bucket=join(analysis_bucket, 'ancestry'),
    #     tmp_bucket=tmp_bucket,
    #     analysis_bucket=analysis_bucket,
    #     relatedness_bucket=relatedness_bucket,
    #     release_related=release_related,
    #     web_bucket=web_bucket,
    #     filter_cutoffs_path=filter_cutoffs_path,
    #     overwrite=overwrite,
    #     scatter_count=scatter_count,
    #     combiner_job=combiner_job,
    #     billing_project=billing_project,
    #     is_test=output_namespace in ['test', 'tmp'],
    #     highmem_workers=highmem_workers,
    # )
    #
    # var_qc_job = add_variant_qc_jobs(
    #     b=b,
    #     work_bucket=join(analysis_bucket, 'variant_qc'),
    #     web_bucket=join(web_bucket, 'variant_qc'),
    #     raw_combined_mt_path=raw_combined_mt_path,
    #     hard_filter_ht_path=hard_filter_ht_path,
    #     meta_ht_path=meta_ht_path,
    #     out_filtered_combined_mt_path=filtered_combined_mt_path,
    #     sample_count=len(samples_df),
    #     ped_file=ped_fpath,
    #     overwrite=overwrite,
    #     vqsr_params_d=utils.get_filter_cutoffs(filter_cutoffs_path)['vqsr'],
    #     scatter_count=scatter_count,
    #     is_test=output_namespace in ['test', 'tmp'],
    #     depends_on=[combiner_job, sample_qc_job],
    #     highmem_workers=highmem_workers,
    # )
    #
    # job_name = 'Remove ref blocks'
    # if not utils.can_reuse(filtered_combined_noref_mt_path, overwrite):
    #     noref_mt_j = add_job(
    #         b,
    #         f'{utils.SCRIPTS_DIR}/make_noref_mt.py '
    #         f'--overwrite '
    #         f'--mt {filtered_combined_mt_path} '
    #         f'--out-mt {filtered_combined_noref_mt_path}',
    #         job_name=job_name,
    #         num_workers=scatter_count,
    #         depends_on=[var_qc_job],
    #     )
    # else:
    #     noref_mt_j = b.new_job(f'{job_name} [reuse]')
    #
    # if subset_projects:
    #     diff_projects = set(subset_projects) - set(input_projects)
    #     if diff_projects:
    #         raise click.BadParameter(
    #             f'--subset-project values should be a subset of --input-project '
    #             f'values. The following projects are not in input projects: '
    #             f'{diff_projects} '
    #         )
    #     subset_projects = list(set(subset_projects))
    #     subset_mt_path = f'{release_bucket}/mt/{output_version}-{"-".join(sorted(subset_projects))}.mt'
    #     job_name = f'Making subset MT for {", ".join(subset_projects)}'
    #     if overwrite or not utils.file_exists(subset_mt_path):
    #         add_job(
    #             b,
    #             f'{utils.SCRIPTS_DIR}/make_subset_mt.py '
    #             f'--mt {filtered_combined_noref_mt_path} ' +
    #             (''.join(f'--subset-project {p} ' for p in subset_projects)) +
    #             f'--out-mt {subset_mt_path}',
    #             job_name=job_name,
    #             is_test=output_namespace in ['test', 'tmp'],
    #             num_workers=scatter_count,
    #             depends_on=[noref_mt_j],
    #         )
    #     else:
    #         b.new_job(f'{job_name} [reuse]')
    #
    #     subset_vcf_path = f'{release_bucket}/vcf/{output_version}-{"-".join(subset_projects)}.vcf.bgz'
    #     job_name = f'Convert subset MT to VCF for {", ".join(subset_projects)}'
    #     if overwrite or not utils.file_exists(subset_vcf_path):
    #         add_job(
    #             b,
    #             f'{utils.SCRIPTS_DIR}/final_mt_to_vcf.py '
    #             f'--mt {subset_mt_path} ' +
    #             f'--out-vcf {subset_vcf_path}',
    #             job_name=job_name,
    #             is_test=output_namespace in ['test', 'tmp'],
    #             num_workers=scatter_count,
    #             preemptible=False,
    #             depends_on=[noref_mt_j],
    #         )
    #     else:
    #         b.new_job(f'{job_name} [reuse]')

    pipeline.run()


if __name__ == '__main__':
    main()



import importlib.metadata
import coloredlogs
from cpg_utils import Path, to_path


coloredlogs.install(
    level='DEBUG', fmt='%(asctime)s %(levelname)s (%(name)s %(lineno)s): %(message)s'
)


def get_package_name() -> str:
    """
    Get name of the package.
    """
    return __name__.split('.', 1)[0]


def get_package_path() -> Path:
    """
    Get local install path of the package.
    """
    return to_path(__file__).parent.absolute()


def get_version() -> str:
    """
    Get package version.
    """
    return importlib.metadata.version(get_package_name())



#!/usr/bin/env python

"""
Convert matrix table to a VCF.
"""

import logging

import click
import hail as hl

from joint_calling.utils import get_validation_callback
from joint_calling import utils, _version

logger = logging.getLogger('joint-calling')
logger.setLevel('INFO')


@click.command()
@click.version_option(_version.__version__)
@click.option(
    '--mt',
    'mt_path',
    required=True,
    callback=get_validation_callback(ext='mt', must_exist=True),
    help='path to split matrix table',
)
@click.option(
    '--out-vcf',
    'out_vcf_path',
    required=True,
    help='path to write VCF',
)
@click.option(
    '--site-only',
    'site_only',
    is_flag=True,
    help='Do not export sample-level information. VCF will not have FORMAT and sample columns'
)
@click.option(
    '--local-tmp-dir',
    'local_tmp_dir',
    help='local directory for temporary files and Hail logs (must be local).',
)
@click.option(
    '--hail-billing',
    'hail_billing',
    help='Hail billing account ID.',
)
def main(
    mt_path: str,
    out_vcf_path: str,
    site_only: bool,
    local_tmp_dir: str,
    hail_billing: str,  # pylint: disable=unused-argument
):  # pylint: disable=missing-function-docstring

    utils.init_hail(__file__, local_tmp_dir)

    mt = hl.read_matrix_table(mt_path)
    write_vcf(mt, out_vcf_path, site_only=site_only)


def write_vcf(mt, out_vcf_path, site_only=False):
    """
    Convert final matrix table to a VCF.
    """
    mt = mt.drop('gvcf_info')

    if site_only:
        ds = mt.rows()
        annotate = ds.annotate
    else:
        ds = mt
        annotate = mt.annotate_rows
    
    ds = annotate(
        info=ds.info.annotate(
            InbreedingCoeff=ds.InbreedingCoeff,
            AC=ds.freq.AC, 
            AF=ds.freq.AF, 
            AN=ds.freq.AN, 
            homozygote_count=ds.freq.homozygote_count,
            faf95=ds.faf.faf95,
            faf99=ds.faf.faf99,
            popmax_AC=ds.popmax.AC, 
            popmax_AF=ds.popmax.AF, 
            popmax_AN=ds.popmax.AN, 
            popmax_homozygote_count=ds.popmax.homozygote_count, 
            popmax_pop=ds.popmax.pop,
            popmax_faf95=ds.popmax.faf95,
            nonsplit_alleles=ds.allele_data.nonsplit_alleles, 
            has_star=ds.allele_data.has_star, 
            variant_type=ds.allele_data.variant_type, 
            n_alt_alleles=ds.allele_data.n_alt_alleles, 
            allele_type=ds.allele_data.allele_type, 
            was_mixed=ds.allele_data.was_mixed, 
        )
    )
    hl.export_vcf(
        ds,
        out_vcf_path,
        tabix=True,
    )


if __name__ == '__main__':
    main()  # pylint: disable=E1120
